@@ -1,5 +1,5 @@
 /******************************************
-Copyright (c) 2016, Mate Soos
+Copyright (C) 2009-2020 Authors of CryptoMiniSat, see AUTHORS file
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -20,14 +20,33 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ***********************************************/
 
-#ifndef __CONSTANTS_H__
-#define __CONSTANTS_H__
+#pragma once
 
 #include <cstdint>
 #include <cstdlib>
 #include <stdio.h>
+#include <limits>
+#include <random>
+
+#define unif_uint_dist(x,y) std::uniform_int_distribution<> x(0, y)
+inline uint32_t rnd_uint(std::mt19937_64& mtrand, const uint32_t maximum) {
+    unif_uint_dist(u, maximum);
+    return u(mtrand);
+}
+
+// #define VERBOSE_DEBUG
+// #define VERBOSE_DEBUG_FULLPROP
+// #define DEBUG_DEPTH
+// #define SLOW_DEBUG
 
 #if defined(_MSC_VER)
+#define cmsat_prefetch(x) (void)(x)
+#else
+#define cmsat_prefetch(x) __builtin_prefetch(x)
+#endif
+
+#if defined(_MSC_VER)
+#include "cms_windows_includes.h"
 #define release_assert(a) \
     do { \
     __pragma(warning(push)) \
@@ -50,15 +69,24 @@ THE SOFTWARE.
     } while (0)
 #endif
 
-#if !defined(__GNUC__) && !defined(__clang__)
-#define __builtin_prefetch(x) (void)(x)
-#endif //__GNUC__
-
 //We shift stuff around in Watched, so not all of 32 bits are useable.
 //for STATS we have 64b values in the Clauses, so they must be aligned to 64
 
 #if defined(STATS_NEEDED)
 #define LARGE_OFFSETS
+#define STATS_DO(x) do {x;} while (0)
+#define INC_ID(cl) \
+    do { \
+        auto prev_id = (cl).stats.ID; \
+        (cl).stats.ID = ++solver->clauseID; \
+        if (solver->sqlStats && (cl).stats.is_tracked) solver->sqlStats->update_id(prev_id, (cl).stats.ID); \
+    } while (0)
+#else
+#define STATS_DO(x) do {} while (0)
+#define INC_ID(cl) \
+    do { \
+        (cl).stats.ID = ++solver->clauseID; \
+    } while (0)
 #endif
 
 #if defined(LARGE_OFFSETS)
@@ -69,6 +97,8 @@ THE SOFTWARE.
 #define EFFECTIVELY_USEABLE_BITS 30
 #endif
 
+#define MAX_XOR_RECOVER_SIZE 8
+
 #if defined _WIN32
     #define DLL_PUBLIC __declspec(dllexport)
 #else
@@ -80,9 +110,51 @@ THE SOFTWARE.
 // Verbose Debug
 ///////////////////
 
-//#define DRAT_DEBUG
+//#define FRAT_DEBUG
 //#define VERBOSE_DEBUG
 
+// verbose print
+#ifdef VERBOSE_DEBUG
+#define VERBOSE_PRINT(x) \
+    do { std::cout << x << std::endl; } while (0)
+#define VERBOSE_DEBUG_DO(x) do { x; } while (0)
+#else
+#define VERBOSE_PRINT(x) do { } while (0)
+#define VERBOSE_DEBUG_DO(x) do { } while (0)
+#endif
+/////
+
+// slow debug
+#ifdef SLOW_DEBUG
+#define SLOW_DEBUG_DO(x) \
+    do { x; } while (0)
+#else
+#define SLOW_DEBUG_DO(x) do { } while (0)
+#endif
+/////
+
+// verb_print
+#define verb_print(a, x) \
+    do { if (solver->conf.verbosity >= a) {std::cout << "c " << x << std::endl;} } while (0)
+/////
+
+// debug watched
+#ifdef DEBUG_WATCHED
+#define DEBUG_WATCHED_DO(x) \
+    do { x; } while (0)
+#else
+#define DEBUG_WATCHED_DO(x) do { } while (0)
+#endif
+
+
+// tbuddy
+#ifdef USE_TBUDDY
+#define TBUDDY_DO(x) \
+    do { x; } while (0)
+#else
+#define TBUDDY_DO(x)  do { } while (0)
+#endif
+
 #ifdef VERBOSE_DEBUG
 #define FAST_DEBUG
 #define DEBUG_ATTACH_FULL
@@ -90,17 +162,6 @@ THE SOFTWARE.
 #define VERBOSE_DEBUG_RECONSTRUCT
 #endif
 
-//Thanks to Axel Kemper for the definitions below
-#ifdef _MSC_VER
-#pragma warning(disable : 4244)  //  C4244 : 'Argument': Konvertierung von 'const uint64_t' in 'double', möglicher Datenverlust
-#pragma warning(disable : 4267)  //  C4267 : 'return': Konvertierung von 'size_t' nach 'uint32_t', Datenverlust möglich
-#pragma warning(disable : 4302)  //  C4302 : truncation
-#pragma warning(disable : 4311)  //  C4311 : pointer truncation
-#pragma warning(disable : 4800)  //  C4800 : 'const uint32_t' : Variable wird auf booleschen Wert('True' oder 'False') gesetzt(Auswirkungen auf Leistungsverhalten möglich)
-#pragma warning(disable : 4805)  //  C4805 : '==' : unsichere Kombination von Typ 'unsigned short' mit Typ 'bool' in einer Operation
-#endif
-
-
 #ifdef __GNUC__
     #define likely(x) __builtin_expect((x), 1)
     #define unlikely(x) __builtin_expect((x), 0)
@@ -138,5 +199,3 @@ THE SOFTWARE.
 #endif
 
 //#define DEBUG_ATTACH_FULL
-
-#endif //__CONSTANTS_H__
